/** \file Cg.h
	Header File - Cg Wrapper classes, init, release

(c) Bastian Goldluecke, MPI Informatik Saarbruecken.
    mailto: bg@mpi-sb.mpg.de

Version history:

\li	\b 0.0 -	06.09.2003 Base implementation

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
<a href="http://www.gnu.org/copyleft/gpl.html">GNU General Public License</a>
for more details.
*/


#ifndef __GOV_CG_PROGRAMS
#define __GOV_CG_PROGRAMS

#include <string>
#include <map>
#include <Cg/cgGL.h>


/// Access to global Cg Objects
/** Global objects are created when this function is first called. They 
    must be destroyed manually after all shaders are deleted using a
    call to releaseCg()
    \ingroup glvp
    \ingroup glfp
*/
class CgObjects;

// Cg-Instance - Funktionen
CgObjects* getCgInstance();
void releaseCg();


/// Global Cg Objects maintenance class
/** The single allowed instance of this object is constructed
    in the first call to Cg() and released using releaseCg().
    \ingroup glfp
    \ingroup glvp
*/
class CgObjects
{
 public:
  bool cgNotSupported;
  std::string lastErrorString;
 
  // Default constructor
  CgObjects();
  // Destructor
  virtual ~CgObjects();
    
  // Access to fragment profile
  CGprofile fragmentProfile();
  const CGprofile fragmentProfile() const;
      
  // Access to vertex profile
  CGprofile vertexProfile();
  const CGprofile vertexProfile() const;
  
  // Access to shader context
  CGcontext shaderContext();
  const CGcontext shaderContext() const;


 private:
  
  // Private error callback function
  static void cgErrorCallback(void);
  
  // Profile for fragment programs
  CGprofile _fragmentProfile;
  // Profile for vertex programs
  CGprofile _vertexProfile;
  // Context for shaders
  CGcontext _shaderContext;
};


/// A Cg program
/** Base class for Cg fragment programs and vertex programs
    \ingroup glfp
    \ingroup glvp
*/
class CgProgram{
 public:
  bool cgNotSupported;
  std::string lastErrorString;
  bool programIsSource;
 
  // Default constructor
  CgProgram();
  // Destructor
  virtual ~CgProgram();
    
  // Set the filename containing the shader source
  void setFilename( const std::string &fn );
  // Initialize program profile
  void initProfile( CGprofile p );

  // Setup, loads program from harddrive
  /** Also performs setup of global objects, if not already initialized */
  virtual int setup();

  // Enable the program
  virtual void enable();
  // Disable the program
  virtual void disable();

  // Read the handle of a parameter of the fragment program
  virtual CGparameter parameter( const std::string &var, int slot = -1 );

  // Set program parameter using parameter ID
  void setParameter( const CGparameter &p, int n, const float *v, bool matrixTranspose );
  //void setParameter( const CGparameter &p, int n, const int *v );
  
	// Set program parameter using parameter name (as in Cg program)
  void setParameter( const std::string &pname, int n, const float *v, bool matrixTranspose = false );
 // void setParameter( const std::string &pname, int n, const int *v );

  // Setzt einen Array Parameter
  void setArrayParameter (const std::string &pname, int n, const float*v, int arraySlot, bool matrixTranspose = false);
  
  bool hasParameter(const std::string& pname);
  
 private:

  // Program code
  CGprogram _Program;
  // Profile the program uses
  CGprofile _Profile;

  // Parameters for the program
  std::map<std::string, CGparameter> _parameters;
  // Filename for the shader source code
  std::string _filename;
};


// A Cg Fragment program
/** \ingroup glfp */
class CgFragmentProgram : public CgProgram
{
 public:
  // Constructor using a file containing the shader source code
  CgFragmentProgram();
  
  // Destructor
  virtual ~CgFragmentProgram();
  
  // Setup, loads fragment program from harddrive
  /** Also performs setup of global objects, if not already initialized */
  int setup();

  // Enable the fragment program and all associated textures
  void enable();
  // Disable the program and all textures
  void disable();

  // Bind a texture handle to a parameter of the fragment program
  void bindTexture( const std::string &var, GLuint T );
  void bindTexture( const std::string &var, GLuint T, int arraySlot );
  
  // Bind a texture handle to a parameter of the fragment program
  void bindTexture( const CGparameter &p, GLuint T  );
  

 private:

  // Textures bound to parameters
  std::map<CGparameter,GLuint> _textures;
};




/// A Cg Vertex program
/** \ingroup glvp */
class CgVertexProgram : public CgProgram
{
 public:
  // Constructor using a file containing the shader source code
  CgVertexProgram();
  // Destructor
  virtual ~CgVertexProgram();

  // Setup the program
  int setup();

  // Set transformation matrix state
  /** Constants for matrices are called
      CG_GL_MODELVIEW_MATRIX, CG_GL_MODELVIEW_PROJECTION_MATRIX, ...
      <br>
      
      Constants for transforms are called CG_GL_MATRIX_IDENTITY, CG_GL_MATRIX_INVERSE, ...
  */
  void setStateMatrixParameter( const CGparameter &p,
				const CGGLenum matrix,
				const CGGLenum transform=CG_GL_MATRIX_IDENTITY ) const;
  
  // Set transformation matrix state
  /** Constants for matrices are called
      CG_GL_MODELVIEW_MATRIX, CG_GL_MODELVIEW_PROJECTION_MATRIX, ...
      <br>

      Constants for transforms are called CG_GL_MATRIX_IDENTITY, CG_GL_MATRIX_INVERSE, ...
  */
  void setStateMatrixParameter( const std::string &pname,
				const CGGLenum matrix,
				const CGGLenum transform=CG_GL_MATRIX_IDENTITY );
};


#endif
